

#include <ap_int.h>
#include <ap_utils.h>
#include <hls_stream.h>

#define IS_CIF963


typedef struct {
	ap_uint<8> data;
	ap_uint<1>  last;
} u8stream;

/**************************************************************************************************
 * THIS WORKS FINE, but needs BRAM optimizations!
 **************************************************************************************************/
void prbs_dbg(hls::stream<u8stream > &input, hls::stream<u8stream > &output, ap_uint<2> *debug);
void prbs(hls::stream<u8stream > &input, hls::stream<u8stream > &output);


// table of matrix products used to update a 32-bit PRBS generator
uint32_t d_prbs_table [4][256];


// table of weights for 8-bit bytes
//    unsigned char d_weight[256];
// PRBS polynomial generator
const uint32_t d_polynomial = 0x110;

/*
 * Generate a parity check for a 32-bit word.
 */
uint32_t parity_check(uint32_t prbs_accum)
{
    uint32_t mask=1UL, parity=0UL;
    int i;
    for (i = 0;  i < 32;  ++i) {
        parity ^= ((prbs_accum & mask) != 0UL);
        mask <<= 1;
    }
    return parity;
}


void gen_prbs_table()
{
    int i;
//    fprintf(stderr, "d_prbs_table[4][256] = {\n");
    int max=0;
    for (i = 0;  i < 4;  ++i) {
//    	fprintf(stderr, "{ ");
        int j;
        for (j = 0;  j < 256;  ++j) {
            uint32_t prbs_accum = ((uint32_t)j << (i * 8));
            int k;
            for (k = 0;  k < 8;  ++k) {
                prbs_accum = (prbs_accum << 1)
                                ^ parity_check(prbs_accum & d_polynomial);
            }
            d_prbs_table[i][j] = (prbs_accum & 0xff);
//            fprintf(stderr, "0x%02x%s ", d_prbs_table[i][j], j != 255 ? "," : "");
//            if(d_prbs_table[i][j] > max) max= d_prbs_table[i][j];
//            fprintf(stderr, "%d%s ", d_prbs_table[i][j], j != 255 ? "," : "");
//            if(j && (j+1) % 16 == 0)
//            	fprintf(stderr, "\n");
        }
//        fprintf(stderr, "}%s\n", i != 3 ? "," : "");
    }
//    fprintf(stderr, "};\nmax=%d\n", max);
}

/*
 * Update a 32-bit PRBS generator eight bits at a time.
 */
#if 0
uint32_t update_prbs(uint32_t d_accum)
{
	uint8_t acc_lsb = 0;
    int i;
    for (i = 0; i < 4; ++i) {
//        PDEBUG("0x%x = 0x%x ^ 0x%x\n",
//            acc_lsb ^ d_prbs_table [i][(d_accum >> (i * 8)) & 0xff],
//            acc_lsb, d_prbs_table [i][(d_accum >> (i * 8)) & 0xff]);
        acc_lsb ^= d_prbs_table[i][(d_accum >> (i * 8)) & 0xff];
    }
    return (d_accum << 8) ^ ((uint32_t)acc_lsb);
}
#else
uint32_t update_prbs(uint32_t d_accum)
{
	uint8_t acc_lsb = d_prbs_table[0][(d_accum) & 0xff];

	if(d_accum & 0x100)
		acc_lsb ^= 0x84;//132;

    return (d_accum << 8) ^ acc_lsb;
}
#endif

int do_prbs(uint8_t *in, int in_size, uint8_t *out, int out_size)
{
	// PRBS accumulator
	uint32_t d_accum = 0;
    while (d_accum < d_polynomial) {
        d_accum <<= 1;
        d_accum |= 1;
    }
//    fprintf(stderr, "d_accum init=0x%08x", d_accum);


    for (int i=0; i < out_size; ++i) {
        // Writting data
        d_accum = update_prbs(d_accum);
        if(in)
        	out[i] = (uint8_t)(d_accum & 0xff) ^ in[i];
        else
        	out[i] = (uint8_t)(d_accum & 0xff);
        //PDEBUG("accum: 0x%x\n", d_accum);
    }
#if 0
    if ((in != NULL) && in_size) {
        fprintf(stderr," mixing input\n");
        if (in_size != out_size) {
        	fprintf(stderr, "%zu != %zu\n", in_size, out_size);
        }
        for (size_t i = 0; i < out_size; ++i) {
            out[i] ^= in[i];
        }
    }
#endif
    return out_size;
}

uint8_t prbs_in_FIC1[96] = {
0x15, 0x02, 0xc2, 0x23, 0x01, 0x00, 0x0e, 0xc2, 0x24, 0x01, 0x00, 0x12, 0xc2, 0x21, 0x01, 0x00,
0x06, 0xc2, 0x36, 0x01, 0x00, 0x1e, 0x04, 0x01, 0x11, 0x34, 0x23, 0xff, 0x00, 0x00, 0xa6, 0x22,
0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0xa8,
0x1d, 0x06, 0xe0, 0x07, 0x22, 0xc2, 0x24, 0xc2, 0x04, 0xe0, 0x09, 0x42, 0xc2, 0x25, 0xc2, 0x05,
0xe0, 0x01, 0x22, 0xc2, 0x21, 0xc2, 0x01, 0xe0, 0x03, 0x22, 0xc2, 0x22, 0xc2, 0x02, 0xd5, 0xd3,
};
uint8_t prbs_out_FIC1[96] = {
0x12, 0xbc, 0xec, 0x47, 0x13, 0x9d, 0xad, 0x0d, 0xbf, 0x14, 0x23, 0x9f, 0x69, 0xa8, 0x89, 0x80,
0x44, 0xf2, 0xaa, 0xaa, 0x0d, 0xf7, 0xbd, 0x15, 0x3a, 0x7b, 0xfa, 0xda, 0xbf, 0x26, 0x00, 0x42,
0xce, 0x94, 0x69, 0x7f, 0x45, 0x8e, 0xb2, 0xcf, 0x1f, 0x74, 0x1a, 0xdb, 0xb0, 0x5a, 0xfa, 0xa8,
0x14, 0xaf, 0x2e, 0xe0, 0x73, 0xa4, 0xf5, 0xd4, 0x48, 0x67, 0x0b, 0xdb, 0x34, 0x3b, 0x6b, 0x56,
0x12, 0x7a, 0xbc, 0xcf, 0x07, 0xf9, 0x63, 0x5d, 0x32, 0xca, 0x4e, 0x59, 0x95, 0x36, 0xd3, 0x05,
0x64, 0x60, 0x1b, 0x94, 0x3a, 0x11, 0x73, 0xc8, 0x55, 0xbd, 0x70, 0x69, 0xbc, 0x4f, 0x99, 0x13,
};
uint8_t prbs_in_FIC2[96] = {
0x06, 0x03, 0x00, 0x10, 0x3c, 0xf8, 0x01, 0x04, 0x01, 0x33, 0x14, 0x15, 0x0d, 0x02, 0xc2, 0x2b,
0x01, 0x00, 0x2e, 0xc2, 0x25, 0x02, 0x00, 0x16, 0x00, 0x20, 0xff, 0x00, 0x00, 0x00, 0xda, 0x41,
0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0xa8,
0x08, 0x06, 0xe0, 0x05, 0x22, 0xc2, 0x23, 0xc2, 0x03, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe2, 0x75,
};
uint8_t prbs_out_FIC2[96] = {
0x01, 0xbd, 0x2e, 0x74, 0x2e, 0x65, 0xa2, 0xcb, 0x9a, 0x26, 0x37, 0x98, 0xa6, 0x8b, 0x4a, 0xab,
0x43, 0x30, 0xb2, 0x69, 0x28, 0xeb, 0xb9, 0x02, 0x2b, 0x6f, 0x26, 0x25, 0xbf, 0x26, 0x7c, 0x21,
0xce, 0x94, 0x69, 0x7f, 0x45, 0x8e, 0xb2, 0xcf, 0x1f, 0x74, 0x1a, 0xdb, 0xb0, 0x5a, 0xfa, 0xa8,
0x14, 0xaf, 0x2e, 0xe0, 0x73, 0xa4, 0xf5, 0xd4, 0x48, 0x67, 0x0b, 0xdb, 0x34, 0x3b, 0x6b, 0x56,
0x07, 0x7a, 0xbc, 0xcd, 0x07, 0xf9, 0x64, 0x5d, 0x35, 0xd5, 0x47, 0x1b, 0x57, 0x13, 0x11, 0x00,
0x84, 0x61, 0x39, 0x56, 0x1b, 0xd3, 0x72, 0x28, 0x56, 0x9f, 0xb2, 0x4b, 0x7e, 0x4d, 0xae, 0xb5,
};



uint8_t prbs_in_CIF1[384] = {
0xff, 0xfc, 0x84, 0x44, 0x2a, 0x62, 0x66, 0x55, 0x55, 0x66, 0x63, 0x22, 0x21, 0x12, 0x48, 0x00,
0x00, 0x00, 0x00, 0xfa, 0xaa, 0x5f, 0xaa, 0xaa, 0xa5, 0xaa, 0x59, 0x95, 0x99, 0x51, 0x45, 0x14,
0x49, 0x25, 0x53, 0x55, 0x39, 0xa1, 0x9a, 0x1a, 0x69, 0xa2, 0x8a, 0x69, 0xaa, 0xaa, 0x28, 0xb2,
0x7b, 0x27, 0xa6, 0x9a, 0x69, 0x37, 0xed, 0x41, 0xbf, 0x6a, 0x09, 0x7a, 0x22, 0x5e, 0x88, 0x28,
0x9c, 0x0a, 0x27, 0x19, 0x16, 0xb2, 0x2d, 0xc3, 0x48, 0xd3, 0xaa, 0x9c, 0x69, 0x48, 0xb0, 0xb6,
0x83, 0x46, 0xf4, 0x1a, 0x37, 0x71, 0x86, 0x5c, 0x61, 0x93, 0xdf, 0x48, 0xf7, 0xd2, 0x6c, 0x22,
0xd8, 0x45, 0x4e, 0x60, 0x8d, 0xc2, 0x47, 0x86, 0x6b, 0x5b, 0x13, 0x91, 0x27, 0x1c, 0x89, 0x3b,
0x89, 0x9a, 0xe2, 0x66, 0xb5, 0xc5, 0xbd, 0x71, 0x6d, 0xb5, 0x83, 0x6b, 0x05, 0xb6, 0x29, 0xb4,
0x8f, 0xa1, 0xc2, 0xd7, 0x22, 0xe4, 0xc4, 0x79, 0x26, 0x23, 0xc7, 0x5d, 0x85, 0xd7, 0x61, 0x7e,
0x1b, 0xdf, 0x86, 0xf8, 0x46, 0xb0, 0x8d, 0x64, 0x91, 0xb7, 0x0f, 0xc7, 0x22, 0xe5, 0x29, 0xac,
0x31, 0x37, 0x79, 0x79, 0xbb, 0xd1, 0x61, 0xd4, 0x58, 0x75, 0x6b, 0x5c, 0x5a, 0xd7, 0x1b, 0x16,
0x36, 0x2c, 0x2a, 0x16, 0xdb, 0x67, 0x4b, 0x13, 0x49, 0x00, 0x0e, 0x42, 0xf1, 0x72, 0x17, 0x8b,
0x55, 0xb8, 0xd5, 0x6e, 0x3b, 0x28, 0xae, 0xca, 0x2b, 0x66, 0x2c, 0xcc, 0x59, 0xf1, 0x7b, 0x72,
0xdb, 0xf4, 0xa1, 0x56, 0x92, 0x38, 0x98, 0x63, 0x44, 0xc3, 0x1a, 0x17, 0x99, 0x85, 0xe6, 0x63,
0x82, 0x58, 0xe0, 0x96, 0x0e, 0x7c, 0x1c, 0xf9, 0x4e, 0x06, 0x26, 0xf0, 0xd2, 0xb5, 0xcc, 0x71,
0xe9, 0x2f, 0x71, 0x49, 0x7b, 0x92, 0x9e, 0xc8, 0xa7, 0xb2, 0xce, 0x8b, 0x73, 0xa2, 0xd6, 0xd3,
0x0d, 0xa6, 0x15, 0x0c, 0xd2, 0xd2, 0xb9, 0x55, 0x16, 0x84, 0x97, 0x8a, 0x72, 0x5c, 0x53, 0x92,
0xed, 0x8d, 0xfb, 0x63, 0x7d, 0xa1, 0x8a, 0x68, 0x62, 0xab, 0x55, 0xd6, 0xab, 0x8b, 0x0c, 0xe4,
0x15, 0x5e, 0xc1, 0x6b, 0x63, 0x92, 0x8b, 0x46, 0xd4, 0x5a, 0x36, 0x66, 0x1a, 0x59, 0x86, 0x96,
0xe8, 0x55, 0xba, 0x15, 0x7b, 0x60, 0xf6, 0xc1, 0x4e, 0x79, 0x89, 0xed, 0x21, 0x64, 0x2d, 0x82,
0xd8, 0x70, 0xb8, 0x43, 0x85, 0xc2, 0x7e, 0xbe, 0x9f, 0xaf, 0xaa, 0x15, 0x3a, 0x84, 0x4d, 0x91,
0x1b, 0x22, 0x39, 0x51, 0xb0, 0xeb, 0xb4, 0x92, 0xb6, 0xcd, 0x55, 0x4e, 0xa7, 0x88, 0x75, 0x3c,
0x4d, 0x9c, 0x47, 0x67, 0x11, 0xca, 0x10, 0xf2, 0x84, 0x31, 0x36, 0xe2, 0x6d, 0xdb, 0x9f, 0x38,
0xc6, 0x0b, 0x5e, 0x36, 0xa9, 0xb5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x99, 0x1e, 0x5c, 0x10, 0x02,
};
uint8_t prbs_out_CIF1[384] = {
0xf8, 0x42, 0xaa, 0x20, 0x38, 0xff, 0xc5, 0x9a, 0xce, 0x73, 0x40, 0xaf, 0x8a, 0x9b, 0xc0, 0x80,
0x42, 0x30, 0x9c, 0x51, 0xa7, 0xb6, 0x13, 0xbe, 0x8e, 0xe5, 0x80, 0xb0, 0x26, 0x77, 0xe3, 0x74,
0x78, 0xb1, 0x3a, 0x2a, 0x7c, 0x2f, 0x28, 0xd5, 0x76, 0xd6, 0x90, 0xb2, 0x1a, 0xf0, 0xd2, 0x1a,
0x6f, 0x88, 0x88, 0x7a, 0x1a, 0x93, 0x18, 0x95, 0xf7, 0x0d, 0x02, 0xa1, 0x16, 0x65, 0x4b, 0xd6,
0x93, 0x76, 0x7b, 0xd1, 0x33, 0x89, 0x6a, 0x5c, 0x7e, 0xf9, 0xed, 0x87, 0x3e, 0x5b, 0xa1, 0xb6,
0x07, 0x27, 0xcd, 0x4c, 0x2c, 0xa2, 0xf4, 0x74, 0x37, 0x0c, 0x6d, 0x03, 0x89, 0x9f, 0x20, 0xe2,
0xbb, 0x6d, 0x9c, 0x9e, 0x06, 0xdf, 0x22, 0x18, 0x55, 0xb3, 0x26, 0x26, 0x47, 0xa9, 0x7c, 0x6b,
0xa0, 0xc4, 0xbf, 0xa6, 0x52, 0x8c, 0x56, 0xd9, 0xfd, 0x7b, 0x94, 0xdd, 0x6d, 0xc1, 0xae, 0x48,
0x91, 0x59, 0x7b, 0x47, 0x68, 0x92, 0x4b, 0x47, 0x4a, 0x77, 0x49, 0x6b, 0x2b, 0xf1, 0x43, 0x7f,
0x13, 0x1d, 0xf4, 0x54, 0x71, 0x16, 0x69, 0x34, 0x3c, 0x88, 0x6b, 0x51, 0xde, 0x7f, 0xb0, 0x2c,
0xf7, 0x66, 0xdc, 0x84, 0xad, 0xeb, 0xaa, 0xe8, 0x25, 0xa5, 0x00, 0x32, 0x9b, 0xbc, 0xf1, 0xb6,
0x64, 0x90, 0x91, 0x97, 0x15, 0xf4, 0x9c, 0x42, 0x68, 0x9c, 0x21, 0x2e, 0x21, 0x9d, 0x18, 0x73,
0x68, 0x49, 0xa6, 0x4e, 0xaf, 0xc5, 0xb0, 0xb6, 0xf3, 0xcf, 0x30, 0xa1, 0x05, 0xbd, 0x3f, 0x70,
0xca, 0x70, 0x44, 0x0e, 0xfd, 0x75, 0x50, 0xc2, 0x1e, 0xbd, 0xd3, 0x3a, 0x60, 0xb0, 0xd5, 0x62,
0x0e, 0xfb, 0xab, 0x6c, 0x22, 0x09, 0x8a, 0x81, 0xb5, 0xa6, 0xf0, 0x2d, 0x50, 0x62, 0x19, 0x31,
0x4c, 0x56, 0x06, 0x4a, 0xe6, 0xb5, 0x30, 0x6a, 0xe4, 0x8a, 0x90, 0x52, 0xd2, 0x7c, 0xc9, 0x23,
0x76, 0x44, 0xf3, 0x4d, 0xfb, 0x08, 0x85, 0xac, 0xa7, 0xd6, 0xaf, 0x50, 0xca, 0xc4, 0xdb, 0x96,
0xce, 0x84, 0x31, 0xd3, 0xa3, 0x3a, 0x1b, 0x2a, 0xd6, 0x56, 0xc7, 0x8d, 0x59, 0xe1, 0x6a, 0xe7,
0x0c, 0x18, 0x56, 0x9f, 0x3b, 0x79, 0xa7, 0xb7, 0x23, 0x1b, 0x9b, 0xdd, 0x1f, 0xf6, 0x2c, 0x17,
0xa2, 0xa7, 0x54, 0x12, 0x41, 0x2f, 0xab, 0x85, 0xc8, 0x09, 0x34, 0x5e, 0x62, 0xd8, 0x12, 0x62,
0x2f, 0xb5, 0x74, 0xc1, 0xd6, 0x76, 0x07, 0x4d, 0xfd, 0x0b, 0xdb, 0xa0, 0x4b, 0xb5, 0x5d, 0x99,
0x5d, 0x31, 0xac, 0x30, 0x0d, 0xdc, 0x96, 0x17, 0xdf, 0x36, 0x71, 0xf9, 0x43, 0x5c, 0xb9, 0x3a,
0x7f, 0x11, 0x68, 0x8f, 0xa0, 0x1c, 0x49, 0x11, 0x6a, 0xb2, 0x6d, 0x94, 0x66, 0x84, 0xca, 0x3a,
0x53, 0xee, 0x82, 0x38, 0xdd, 0x2b, 0xba, 0x89, 0x0c, 0xe1, 0x7b, 0xff, 0x99, 0x24, 0x6f, 0xc3,
};



uint8_t prbs_in_CIF2[240] = {
0xff, 0xfc, 0x54, 0xc4, 0xd5, 0xd5, 0x65, 0x34, 0x33, 0x12, 0x21, 0x12, 0x49, 0x00, 0x00, 0x00,
0x00, 0x85, 0x8b, 0xaa, 0xa9, 0x27, 0x61, 0x7a, 0x38, 0x66, 0x92, 0x59, 0xa5, 0x8a, 0x6a, 0x66,
0x9e, 0x7a, 0x27, 0xa2, 0x69, 0x9f, 0xc0, 0x7e, 0xc4, 0x86, 0xa3, 0xa9, 0xa2, 0xdb, 0x89, 0x2b,
0x99, 0xc8, 0x61, 0x94, 0x63, 0x61, 0xbd, 0x79, 0x0a, 0x91, 0xa8, 0x0c, 0x2d, 0x5a, 0x52, 0xb3,
0x5f, 0x6b, 0x4d, 0xc4, 0x69, 0xde, 0xcd, 0xa2, 0xdc, 0xf3, 0xbb, 0xa3, 0xc1, 0x12, 0x6d, 0xb8,
0x8a, 0x6a, 0xa5, 0x93, 0x68, 0xdf, 0x34, 0x57, 0x40, 0x6f, 0xa5, 0x41, 0x02, 0xe2, 0x33, 0x30,
0x03, 0xd8, 0x3e, 0x02, 0x80, 0xd5, 0xd3, 0xc0, 0x4f, 0x1e, 0xc5, 0x53, 0x8e, 0x5a, 0xc9, 0x35,
0xf5, 0x68, 0x09, 0xa2, 0xa5, 0xa4, 0x35, 0x4a, 0xb2, 0xc6, 0x14, 0xcb, 0x9f, 0x11, 0x8b, 0x15,
0xa0, 0xdb, 0x51, 0x59, 0x39, 0x4f, 0x3c, 0xc4, 0xf4, 0xb3, 0x34, 0x50, 0xe6, 0xeb, 0x99, 0xb6,
0x24, 0x2b, 0x5d, 0x84, 0x6b, 0xae, 0x39, 0x53, 0x2e, 0x24, 0x35, 0x52, 0x38, 0xd4, 0x6d, 0xcc,
0x87, 0x1b, 0xba, 0xed, 0x6a, 0x4e, 0x2b, 0xb9, 0x0a, 0xaa, 0x1c, 0xf5, 0x17, 0x35, 0x8f, 0x35,
0xf6, 0x1c, 0xb5, 0xb6, 0x7a, 0x73, 0x3e, 0x51, 0xcd, 0x6b, 0xab, 0xbb, 0x76, 0xe3, 0x69, 0xba,
0x2c, 0xc7, 0xb9, 0x25, 0x74, 0x0c, 0xa0, 0xd1, 0x60, 0xf6, 0x9c, 0xaa, 0xfc, 0xa2, 0x8a, 0x35,
0x4b, 0x66, 0x5a, 0xca, 0x6b, 0x7e, 0x48, 0x50, 0xf9, 0xd2, 0x3d, 0x54, 0x30, 0xea, 0x70, 0x3b,
0xa2, 0x0a, 0xd5, 0x96, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x71, 0x04, 0xc3, 0x11, 0x22,
};
uint8_t prbs_out_CIF2[240] = {
0xf8, 0x42, 0x7a, 0xa0, 0xc7, 0x48, 0xc6, 0xfb, 0xa8, 0x07, 0x02, 0x9f, 0xe2, 0x89, 0x88, 0x80,
0x42, 0xb5, 0x17, 0x01, 0xa4, 0xce, 0xd8, 0x6e, 0x13, 0x29, 0x4b, 0x7c, 0x1a, 0xac, 0xcc, 0x06,
0xaf, 0xee, 0x4e, 0xdd, 0x2c, 0x11, 0x72, 0xb1, 0xdb, 0xf2, 0xb9, 0x72, 0x12, 0x81, 0x73, 0x83,
0x8d, 0x67, 0x4f, 0x74, 0x10, 0xc5, 0x48, 0xad, 0x42, 0xf6, 0xa3, 0xd7, 0x19, 0x61, 0x91, 0x4d,
0x50, 0x17, 0x11, 0x0c, 0x4c, 0xe5, 0x8a, 0x3d, 0xea, 0xd9, 0xfc, 0xb8, 0x96, 0x01, 0x7c, 0xb8,
0x0e, 0x0b, 0x9c, 0xc5, 0x73, 0x0c, 0x46, 0x7f, 0x16, 0xf0, 0x17, 0x0a, 0x7c, 0xaf, 0x7f, 0xf0,
0x60, 0xf0, 0xec, 0xfc, 0x0b, 0xc8, 0xb6, 0x5e, 0x71, 0xf6, 0xf0, 0xe4, 0xee, 0xef, 0x3c, 0x65,
0xdc, 0x36, 0x54, 0x62, 0x42, 0xed, 0xde, 0xe2, 0x22, 0x08, 0x03, 0x7d, 0xf7, 0x66, 0x0c, 0xe9,
0xbe, 0x23, 0xe8, 0xc9, 0x73, 0x39, 0xb3, 0xfa, 0x98, 0xe7, 0xba, 0x66, 0x48, 0xcd, 0xbb, 0xb7,
0x2c, 0xe9, 0x2f, 0x28, 0x5c, 0x08, 0xdd, 0x03, 0x83, 0x1b, 0x51, 0xc4, 0xc4, 0x4e, 0xf4, 0x4c,
0x41, 0x4a, 0x1f, 0x10, 0x7c, 0x74, 0xe0, 0x85, 0x77, 0x7a, 0x77, 0x9b, 0xd6, 0x5e, 0x65, 0x95,
0xa4, 0xa0, 0x0e, 0x37, 0xb4, 0xe0, 0xe9, 0x00, 0xec, 0xf7, 0x84, 0xd7, 0xa6, 0x0c, 0x66, 0x42,
0x11, 0x36, 0xca, 0x05, 0xe0, 0xe1, 0xbe, 0xad, 0xb8, 0x5f, 0x80, 0xc7, 0xa0, 0xee, 0xce, 0x37,
0x5a, 0xe2, 0xbf, 0x92, 0x04, 0x33, 0x80, 0xf1, 0xa3, 0xac, 0xf4, 0x79, 0xc9, 0xdf, 0x43, 0x3a,
0x2e, 0xa9, 0x9e, 0x6c, 0x44, 0x75, 0x96, 0x78, 0xfb, 0xa0, 0xd6, 0xac, 0x86, 0x14, 0xc4, 0x62,
};




int do_test(uint8_t *input, int input_size, uint8_t *output, int output_size)
{

	int not_ok = 0;
	/**************************************************************************************
	 * HW Implementation
	 **************************************************************************************/
    hls::stream<u8stream> data_in;
    hls::stream<u8stream> data_out;

    u8stream var;
    int rules_num=0;

    uint8_t *out2 = (uint8_t *) calloc(1, output_size);

    do_prbs(input, input_size, out2, output_size);

    if(memcmp(output, out2, output_size)) {
    	fprintf(stderr, "[SOFT] mem is different:\n");
    	for(int i =0; i< output_size; i++) {
    		if(output[i] != out2[i])
    			fprintf(stderr, "[SOFT] mem[%d]: %02x != %02x\n", i, output[i], out2[i]);
    	}
        return 1;
    } else {
    	fprintf(stderr, "[SOFT] mem ok\n");
//        return 0;
    }


  memset(out2, 0x00, output_size);

    ap_uint<2> debug = 0;

	var.last = 0;
	var.data = 0xF1; //pass to next block
	data_in.write(var);

	//SEND FC
//1st FIC:			{0x66, 0x8c, 0x8b, 0x85};
	var.data = 0x9b;
	data_in.write(var);
	var.data = 0x8c;
	data_in.write(var);
	var.data = 0x2b;
	data_in.write(var);
	var.data = 0x85;
	data_in.write(var);


	for(int i=0;i<input_size;i++) {
		var.data = input[i];
		var.last = (i==(input_size-1)) ? 1 : 0;
		data_in.write(var);
	}

	prbs_dbg(data_in, data_out, &debug);

	int read_size = data_out.size();
	if(read_size > 2) {
		fprintf(stderr, "[FIC HARD] PRBS read[%d]: \n", read_size);
		u8stream var2;
		var2=data_out.read();
		fprintf(stderr, "[FIC HARD] PRBS CIF/FIC id: 0x%02x\n", (uint8_t) var2.data);
		read_size-=1;

#if 1
		fprintf(stderr, "[FIC HARD] PRBS FC = 0x");

		for(int y=0; y<4;y++) {
			var2=data_out.read();
			fprintf(stderr, "%02x", (uint8_t) var2.data);
			read_size-=1;
		}
		fprintf(stderr, "\n");
#endif

		int readed=0;
		do {
			var2=data_out.read();
			//fprintf(stderr, "[%02x]%s, ", (int)var2.data, var2.last ? "L":"");
			//if((readed+1)%16==0) fprintf(stderr, "\n");
			read_size--;

			if(readed<output_size)
				out2[readed] = var2.data;
			else if(var2.data == 0xFA) {
				fprintf(stderr, "Got phase reference filler:\n");
				for(int y=0; y<4;y++) {
					var2=data_out.read();
					if((int)var2.data != 0x00 || var2.last) fprintf(stderr, "WRONG PRBS NULL: [%02x]%s, ", (int)var2.data, var2.last ? "L":"");

					read_size--;
				}

				for(int y=0; y<6912;y++) {
					var2=data_out.read();
					//fprintf(stderr, "[%02x]%s, ", (int)var2.data, var2.last ? "L":"");
					//if((y+1)%16==0) fprintf(stderr, "\n");
					//fprintf(stderr, "%02x", (uint8_t) var2.data);
					read_size--;
				}
				if(var2.last != 1)
					fprintf(stderr, "ERROR: HAVE MORE THAN NEEDED????\n");
			} else {
				fprintf(stderr, "UNNEEDED!!!\n");
			}

			readed++;
		} while (read_size);
		fprintf(stderr, "\nreaded total: %d\n", readed);
	}

	if(memcmp(output, out2, output_size)) {
	    	fprintf(stderr, "[FIC HARD] mem is different:\n");
	    	for(int i =0; i< sizeof(out2); i++) {
	    		if(output[i] != out2[i])
	    			fprintf(stderr, "[FIC HARD] mem[%d]: %02x != %02x\n", i, output[i], out2[i]);
	    	}
	    	free(out2);
	        return 1;
	} else {
		fprintf(stderr, "[FIC HARD] mem ok\n");
		free(out2);
		return 0;
	}
}






int main()
{
	int ret=1;

	 gen_prbs_table();
	 ret = do_test(prbs_in_FIC1, sizeof(prbs_in_FIC1), prbs_out_FIC1, sizeof(prbs_out_FIC1));
	 if(ret)
		 return ret;

	 ret = do_test(prbs_in_FIC2, sizeof(prbs_in_FIC2), prbs_out_FIC2, sizeof(prbs_out_FIC2));
	 if(ret)
		 return ret;

	 ret = do_test(prbs_in_CIF1, sizeof(prbs_in_CIF1), prbs_out_CIF1, sizeof(prbs_out_CIF1));
	 if(ret)
		 return ret;

	 ret = do_test(prbs_in_CIF2, sizeof(prbs_in_CIF2), prbs_out_CIF2, sizeof(prbs_out_CIF2));
	 if(ret)
		 return ret;

	return ret;
}

